# -*- coding: utf-8 -*-
import os, sys, json, functools, shutil, re
from maya import cmds, mel
from PySide2 import QtCore, QtGui, QtWidgets
from shiboken2 import wrapInstance
import maya.OpenMayaUI as omui

try:
    import _winreg as regedit
except:
    import winreg as regedit



def mayaMainWindows():
    mainWindowsPtr = omui.MQtUtil.mainWindow()
    return wrapInstance(int(mainWindowsPtr),QtWidgets.QWidget)
def get_douments():
    key = regedit.OpenKey(regedit.HKEY_CURRENT_USER,r"Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders")
    value = regedit.QueryValueEx(key, "Personal")
    return value[0].replace("\\","/")


def printattr(obj):
    if cmds.objExists(obj):
        allattr_dict={}
        shortname={}
        attrlist=[]
        for item in cmds.listAttr(obj):
            try:
                allattr_dict[item]=cmds.getAttr(obj+'.'+item)
                shortname[item]=cmds.listAttr((obj+'.'+item),sn=1)[0]
                attrlist.append(item)
            except:
                pass
        w = max(map(len, allattr_dict.keys())) 
        n = max(map(len, shortname.values())) 
        attrlist=sorted(attrlist)
        for k in attrlist:
            print(k.ljust(w), '->',shortname[k].ljust(n) , '->',allattr_dict[k])


class myListWidget(QtWidgets.QListWidget):
    draged = QtCore.Signal()
    def __init__(self, parent = None):
        super(myListWidget,self).__init__(parent)

    def dragEnterEvent(self,event):
        super(myListWidget,self).dragEnterEvent(event)
        self.draged.emit()
    def dropEvent(self, event):
        super(myListWidget,self).dropEvent(event)
        self.draged.emit()

class Default_Setting(QtWidgets.QDialog):
    def __init__(self, title, json_path, json_dic, parent=mayaMainWindows()):
        QtWidgets.QDialog.__init__(self,parent)
        # 
        self.parent = parent
        self.return_bool = False
        self.title = title
        self.json_path = json_path
        self.json_dic = json_dic
        self.load_json_dic()

        # 创建UI，更改显示文字
        self.setupUi(self)
        self.show_default_setting()
        # 显示UI
        self.show()
    def setupUi(self, Form):
        Form.resize(375, 397)
        self.verticalLayout = QtWidgets.QVBoxLayout(Form)
        self.tableWidget = QtWidgets.QTableWidget(Form)
        self.verticalLayout.addWidget(self.tableWidget)

        QtCore.QMetaObject.connectSlotsByName(Form)
                
        # tablewidget右键
        self.tableWidget.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tableWidget.customContextMenuRequested.connect(self.show_custom_menu)
        self.create_right_menu()
        #
        self.tableWidget.verticalHeader().setSectionResizeMode(QtWidgets.QHeaderView.ResizeToContents)
        self.tableWidget.horizontalHeader().setSectionResizeMode( QtWidgets.QHeaderView.ResizeToContents)


        self.setWindowFlags(QtCore.Qt.Window)
        self.setWindowModality(QtCore.Qt.ApplicationModal)
        self.setWindowTitle(self.title)
    # 右键
    def create_right_menu(self):
        add_icon = QtGui.QIcon()
        add_icon.addPixmap(QtGui.QPixmap(":/UVTBAdd.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        remove_icon = QtGui.QIcon()
        remove_icon.addPixmap(QtGui.QPixmap(":/UVTBRemove.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        save_icon = QtGui.QIcon()
        save_icon.addPixmap(QtGui.QPixmap(":/save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        open_icon = QtGui.QIcon()
        open_icon.addPixmap(QtGui.QPixmap(":/fileOpen.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        trash_icon = QtGui.QIcon()
        trash_icon.addPixmap(QtGui.QPixmap(":/trash.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

        self.RightMenu = QtWidgets.QMenu(self)
        menuitemA = self.RightMenu.addAction(add_icon, u'添加行')
        menuitemA.triggered.connect(self.add_table_rowcount)
        if self.title == u'设置命名预设':
            menuitemB = self.RightMenu.addAction(add_icon, u'添加列')
            menuitemB.triggered.connect(self.add_table_columncount)

            menuitemE = self.RightMenu.addAction(u'设置标签')
            menuitemE.triggered.connect(self.set_row_label)

            menuitemD = self.RightMenu.addAction(trash_icon,u'删除')
            menuitemD.triggered.connect(self.delete_row)

            menuitemC = self.RightMenu.addAction(open_icon,u'选择预设')
            menuitemC.triggered.connect(self.select_name_setting)

        self.RightMenu.addSeparator()

        # menuitemB.triggered.connect(functools.partial(self.add_gpucache_node,"gpu"))
    def show_custom_menu(self):
        self.RightMenu.exec_(QtGui.QCursor().pos())
    # 关闭窗口事件
    def closeEvent(self, event):
        # ui = QtWidgets.QMessageBox.question(self, u"Question", u"是否保存设置??", QtWidgets.QMessageBox.Yes|QtWidgets.QMessageBox.No)
        # if ui == QtWidgets.QMessageBox.Yes:
        self.save_json_dic()
        self.return_bool = True
    # 读取json文件
    def load_json_dic(self):
        filepath = self.json_path.rsplit("/",1)[0]
        if not os.path.exists(filepath):
            os.makedirs(filepath)
        
        if os.path.exists(self.json_path):
            with open(self.json_path,"r") as f:
                self.json_dic = json.load(f)
        else:
            with open(self.json_path,"w") as f:
                json.dump(self.json_dic,f,indent=4)
    # 设置表格行列
    def set_widget_row_col(self,row=None,column=None):
        if column:
            self.tableWidget.setColumnCount(column)
        if row:
            self.tableWidget.setRowCount(row)
    # 设置表格某一行的内容
    def set_table_row(self, row, string_list):
        column = 0
        for item in string_list:
            if item:
                newItem=QtWidgets.QTableWidgetItem(str(item))
                self.tableWidget.setItem(row, column, newItem)
            column += 1
    # 返回最长的list的len
    def return_dic_long_list_len(self,_dic):
        list_len = 0
        for k,v in _dic.items():
            if len(v) > list_len:
                list_len = len(v)
        return list_len
    # 显示json的内容
    def show_default_setting(self):
        row = 0
        if self.title == u"设置贴图节点与属性":
            self.set_widget_row_col(row = len(self.json_dic), column = 2)
            self.tableWidget.setHorizontalHeaderLabels([u"节点类型", u"节点属性名称"])

            for k,v in self.json_dic.items():
                self.set_table_row(row, [k,v])
                row += 1
        elif self.title == u"设置材质球类型":
            self.set_widget_row_col(row = len(self.json_dic["shader"]), column = 1)
            self.tableWidget.setHorizontalHeaderLabels([u"材质球类型"])
            for i in self.json_dic["shader"]:
                self.set_table_row(row, [i])
                row += 1

        elif self.title == u"设置属性转命名":
            self.set_widget_row_col(row = len(self.json_dic), column = 2)
            self.tableWidget.setHorizontalHeaderLabels([u"材质属性", u"替换命名"])
            
            for k,v in self.json_dic.items():
                self.set_table_row(row, [k,v])
                row += 1
        elif self.title == u"设置命名预设":
            self.set_widget_row_col(row = len(self.json_dic), column = self.return_dic_long_list_len(self.json_dic))
            
            for k,v in self.json_dic.items():
                newItem=QtWidgets.QTableWidgetItem(k)
                self.tableWidget.setVerticalHeaderItem(row, newItem)

                self.set_table_row(row, v)
                row += 1


    # 保存配置信息
    def save_json_dic(self):
        if self.title == u"设置贴图节点与属性" or self.title == u"设置属性转命名":
            save_dic = {}
            for row in range(self.tableWidget.rowCount()):
                _list = []
                for col in range(self.tableWidget.columnCount()):
                    widgetItem = self.tableWidget.item(row, col)
                    if widgetItem and widgetItem.text() != "":
                        _list.append(widgetItem.text())
                if len(_list) == 2:
                    save_dic[_list[0]] = _list[1]
            self.json_dic = save_dic
        elif self.title == u"设置材质球类型":
            _list = []
            for row in range(self.tableWidget.rowCount()):
                for col in range(self.tableWidget.columnCount()):
                    widgetItem = self.tableWidget.item(row, col)
                    if widgetItem and widgetItem.text() != "":
                        _list.append(widgetItem.text())

            self.json_dic["shader"] = list(set(_list))
        elif self.title == u"设置命名预设":
            save_dic = {}
            for row in range(self.tableWidget.rowCount()):
                _list = []
                labelitem = self.tableWidget.verticalHeaderItem(row)
                if labelitem:
                    label = labelitem.text()
                    for col in range(self.tableWidget.columnCount()):
                        widgetItem = self.tableWidget.item(row, col)
                        if widgetItem:
                            _list.append(widgetItem.text())
                        save_dic[label] = _list
            self.json_dic = save_dic

        # 保存json
        with open(self.json_path,"w") as f:
            json.dump(self.json_dic,f,indent=4)

    # 右键执行
    def add_table_rowcount(self):
            i, okPressed = QtWidgets.QInputDialog.getInt(self, u"输入增加表格行数",u"行数：", 5, 0, 100, 1)
            if okPressed and i:
                self.tableWidget.setRowCount(self.tableWidget.rowCount()+i)
    def add_table_columncount(self):
            i, okPressed = QtWidgets.QInputDialog.getInt(self, u"输入增加表格列数",u"行数：", 1, 0, 100, 1)
            if okPressed and i:
                self.tableWidget.setColumnCount(self.tableWidget.columnCount()+i)
    def set_row_label(self):
        row = self.tableWidget.currentRow()
        text, okPressed = QtWidgets.QInputDialog.getText(self, u"新建标签",u"名字:", QtWidgets.QLineEdit.Normal, "")
        if okPressed and text != '':
            newItem=QtWidgets.QTableWidgetItem(text)
            self.tableWidget.setVerticalHeaderItem(row,newItem)

        
    # 删除行
    def delete_row(self):
        row = self.tableWidget.currentRow()
        self.tableWidget.removeRow(row)
    # 选择预设
    def select_name_setting(self):
        selItem = self.tableWidget.selectedItems()
        if selItem:
            row = selItem[0].row()
            self.parent.name_listwidget.clear()
            for col in range(self.tableWidget.columnCount()):
                itemname = self.tableWidget.item(row, col)
                if itemname:
                    self.parent.name_listwidget.addItem(itemname.text())
                    self.parent.refresh_name_label()

    @staticmethod
    def Set_setting_info(title, json_path, json_dic, parent_widget):
        ui = Default_Setting(title, json_path, json_dic, parent_widget)
        ui.exec_()
        return True


        
class RenameShaderWindow(object):
    def __init__(self):
        # makedir
        scirpt_dir = get_douments() + "/maya/rename_shader_textures"
        if not os.path.exists(scirpt_dir):
            os.makedirs(scirpt_dir)
        # json文件
        self.node_fileattr_josn = get_douments() + "/maya/rename_shader_textures/node_fileattr.json"
        self.shader_nodetype_josn = get_douments() + "/maya/rename_shader_textures/shader_nodetype.json"
        self.attr_filename_json = get_douments() + "/maya/rename_shader_textures/attr_filename.json"
        self.new_name_set_json = get_douments() + "/maya/rename_shader_textures/new_name_set.json"

        self.node_fileattr = {"file":".ftn"}
        self.shader_nodetype = {"shader":["aiStandardSurface",
                                            "lambert",
                                            "phongE",
                                            "blinn",
                                            "aiLayerShader",
                                            "phong",
                                            "layeredShader"
                                            ]}
        self.attr_filename =   {"baseColor":"BaseColor",
                                "specularColor":"Specular",
                                "emissionColor":"Emission",
                                "normalCamera":"Bump",
                                "aiMatteColor":"Matte",
                                }
        self.new_name_set = {}
        
        self.save_filepath = None # 贴图另存的文件路径
        self.filenode_oldname = {} # 双击记录节点名和属性
    def setupUi(self, RenameShaderWindow):
        RenameShaderWindow.setObjectName("RenameShaderWindow")
        RenameShaderWindow.resize(727, 600)
        self.centralwidget = QtWidgets.QWidget(RenameShaderWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.splitter = QtWidgets.QSplitter(self.centralwidget)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.widget = QtWidgets.QWidget(self.splitter)
        self.widget.setObjectName("widget")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.widget)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.attr_tablewidget = QtWidgets.QTableWidget(self.widget)
        self.attr_tablewidget.setObjectName("attr_tablewidget")
        self.attr_tablewidget.setColumnCount(0)
        self.attr_tablewidget.setRowCount(0)
        self.verticalLayout_3.addWidget(self.attr_tablewidget)
        self.get_mesh_filepath_button = QtWidgets.QPushButton(self.widget)
        self.get_mesh_filepath_button.setObjectName("get_mesh_filepath_button")
        self.verticalLayout_3.addWidget(self.get_mesh_filepath_button)
        self.groupBox = QtWidgets.QGroupBox(self.splitter)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.texture_label = QtWidgets.QLabel(self.groupBox)
        self.texture_label.setObjectName("texture_label")
        self.verticalLayout.addWidget(self.texture_label)
        self.pic_label = QtWidgets.QLabel(self.groupBox)
        self.pic_label.setObjectName("pic_label")
        self.verticalLayout.addWidget(self.pic_label)
        self.name_listwidget = myListWidget(self.groupBox)
        self.name_listwidget.setObjectName("name_listwidget")
        self.verticalLayout.addWidget(self.name_listwidget)
        self.groupBox_2 = QtWidgets.QGroupBox(self.groupBox)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.widget_3 = QtWidgets.QWidget(self.groupBox_2)
        self.widget_3.setObjectName("widget_3")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.widget_3)
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.pic_path_radioButton = QtWidgets.QRadioButton(self.widget_3)
        self.pic_path_radioButton.setChecked(True)
        self.pic_path_radioButton.setObjectName("pic_path_radioButton")
        self.horizontalLayout_3.addWidget(self.pic_path_radioButton)
        self.new_path_radioButton = QtWidgets.QRadioButton(self.widget_3)
        self.new_path_radioButton.setObjectName("new_path_radioButton")
        self.horizontalLayout_3.addWidget(self.new_path_radioButton)
        self.verticalLayout_2.addWidget(self.widget_3)
        self.widget_2 = QtWidgets.QWidget(self.groupBox_2)
        self.widget_2.setEnabled(True)
        self.widget_2.setObjectName("widget_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.widget_2)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.path_lineedit = QtWidgets.QLineEdit(self.widget_2)
        self.path_lineedit.setObjectName("path_lineedit")
        self.horizontalLayout_2.addWidget(self.path_lineedit)
        self.get_path_button = QtWidgets.QPushButton(self.widget_2)
        self.get_path_button.setMinimumSize(QtCore.QSize(30, 0))
        self.get_path_button.setMaximumSize(QtCore.QSize(30, 16777215))
        self.get_path_button.setObjectName("get_path_button")
        self.horizontalLayout_2.addWidget(self.get_path_button)
        self.verticalLayout_2.addWidget(self.widget_2)
        self.apply_button = QtWidgets.QPushButton(self.groupBox_2)
        self.apply_button.setObjectName("apply_button")
        self.verticalLayout_2.addWidget(self.apply_button)
        self.verticalLayout.addWidget(self.groupBox_2)
        self.horizontalLayout.addWidget(self.splitter)
        RenameShaderWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(RenameShaderWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 727, 23))
        self.menubar.setObjectName("menubar")
        self.menuEdit = QtWidgets.QMenu(self.menubar)
        self.menuEdit.setObjectName("menuEdit")
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        RenameShaderWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(RenameShaderWindow)
        self.statusbar.setObjectName("statusbar")
        RenameShaderWindow.setStatusBar(self.statusbar)
        self.menubar.addAction(self.menuEdit.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(RenameShaderWindow)
        QtCore.QMetaObject.connectSlotsByName(RenameShaderWindow)

        #--------------------------------------------------------------------------------------------------ui设置
        # 菜单
        self.create_menu_bar()
        # tablewidget设置
        self.attr_tablewidget.setSortingEnabled(True)
        self.attr_tablewidget.setColumnCount(5)
        self.attr_tablewidget.verticalHeader().setSectionResizeMode(QtWidgets.QHeaderView.ResizeToContents)
        self.attr_tablewidget.horizontalHeader().setSectionResizeMode( QtWidgets.QHeaderView.ResizeToContents)
        #tablewidget 右键
        self.attr_tablewidget.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.attr_tablewidget.customContextMenuRequested.connect(self.show_custom_menu)
        self.create_right_menu()
        #listwidget 右键
        self.name_listwidget.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.name_listwidget.customContextMenuRequested.connect(self.show_listwidget_right_menu)
        self.create_listwidget_right_menu()
        #listwidget 多选
        self.name_listwidget.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        #listwidget 字体大小
        font = QtGui.QFont()
        font.setPointSize(12)
        self.name_listwidget.setFont(font)
        #listwidget 拖拽移动item
        self.name_listwidget.setDragDropMode(QtWidgets.QAbstractItemView.InternalMove)
        # 隐藏路径
        self.path_lineedit.setEnabled(False)
        self.widget_2.setVisible(False)
        #设置窗口flags
        self.setWindowFlags(QtCore.Qt.Window)
        self.ui_ctrl()
    def retranslateUi(self, RenameShaderWindow):
        RenameShaderWindow.setWindowTitle(u"贴图重命名工具 v1.09")
        self.get_mesh_filepath_button.setText(u"获取选择模型的贴图")
        self.groupBox.setTitle(u"详细命名")
        self.texture_label.setText(u"贴图命名：(文件后缀自动识别)")
        self.pic_label.setText(u"项目_角色_部位_属性.png")
        self.groupBox_2.setTitle(u"开始重命名")
        self.pic_path_radioButton.setText(u"图片所在路径")
        self.new_path_radioButton.setText(u"新路径")
        self.get_path_button.setText(u"...")
        self.apply_button.setText(u"Apply")
        self.menuEdit.setTitle(u"Edit")
        self.menuHelp.setTitle(u"Help")

        self.attr_tablewidget.setHorizontalHeaderLabels([u"节点名称", u"贴图路径", u"链接属性", u"属性->命名", u"新命名"])

#--------------------------------------------------------------------------------------------------信号
    def ui_ctrl(self):
        self.attr_tablewidget.itemDoubleClicked.connect(self.get_doubleclick_content)
        self.attr_tablewidget.itemChanged.connect(self.rename_filenode_and_attr)

        self.get_mesh_filepath_button.clicked.connect(self.get_sel_textures)
        self.name_listwidget.draged.connect(self.refresh_name_label)
        
        self.pic_path_radioButton.toggled.connect(self.change_path_show)
        self.new_path_radioButton.toggled.connect(self.change_path_show)
        self.get_path_button.clicked.connect(self.change_line_edit)

        self.apply_button.clicked.connect(self.start_rename)
    # 关闭窗口事件
    def closeEvent(self, event):
        self.save_json_dic()
#--------------------------------------------------------------------------------------------------ui追加
    # 更新提示
    def update_title(self, num):
        try:
            import requests
            req = requests.get("https://family-xiaojing-1256018034.cos.ap-guangzhou.myqcloud.com/%E8%84%9A%E6%9C%AC%E5%88%86%E4%BA%AB/rename_shader_textures.txt")
            print(req.content)
            if num < float(req.content):
                self.setWindowTitle(u"贴图重命名工具 v1.09  {}".format(u"有新版本，请更新"))
        except:
            self.setWindowTitle(u"贴图重命名工具 v1.09  {}".format(u"获取更新失败，请检查网络"))
    # 菜单
    def create_menu_bar(self):
        edit_itemA = self.menuEdit.addAction(u'设置贴图节点与属性')
        edit_itemB = self.menuEdit.addAction(u'设置材质球类型')
        edit_itemC = self.menuEdit.addAction(u'设置属性转命名')
        edit_itemD = self.menuEdit.addAction(u'命名预设')
        
        help_itemA = self.menuHelp.addAction(u'显示帮助')
        help_itemB = self.menuHelp.addAction(u'获取选择节点类型')
        help_itemC = self.menuHelp.addAction(u'获取选择节点属性')

        edit_itemA.triggered.connect(functools.partial(self.show_setting_win, u'设置贴图节点与属性', self.node_fileattr_josn, self.node_fileattr, self))
        edit_itemB.triggered.connect(functools.partial(self.show_setting_win, u'设置材质球类型', self.shader_nodetype_josn, self.shader_nodetype, self))
        edit_itemC.triggered.connect(functools.partial(self.show_setting_win, u'设置属性转命名', self.attr_filename_json, self.attr_filename, self))
        edit_itemD.triggered.connect(functools.partial(self.show_setting_win, u'设置命名预设', self.new_name_set_json, self.new_name_set, self))

        help_itemA.triggered.connect(self.open_blog)
        help_itemB.triggered.connect(self.print_objtype)
        help_itemC.triggered.connect(self.print_attr)
    
    # 打开博客
    def open_blog(self):
        import webbrowser
        webbrowser.open("http://smilejing.cn/archives/338")
    # 节点类型
    def print_objtype(self):
        if cmds.ls(sl=1):
            for i in cmds.ls(sl=1):
                print(cmds.objectType(i))
    # 节点属性
    def print_attr(self):
        if cmds.ls(sl=1):
            for i in cmds.ls(sl=1):
                print("-----------------------",i,"-----------------------")
                printattr(i)
    #tablewidget 右键
    def create_right_menu(self):
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/futurePulldownIcon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

        self.tablewidgetRightMenu = QtWidgets.QMenu(self)
        menuitemA = self.tablewidgetRightMenu.addAction(icon, u'获取链接属性')
        self.tablewidgetRightMenu.addSeparator()
        # menuitemB = self.tablewidgetRightMenu.addAction(u'添加GPUcache(实模形态)')
        # self.tablewidgetRightMenu.addSeparator()
        # menuitemC = self.tablewidgetRightMenu.addAction(u'添加GPUcache(BBox形态)')

        menuitemA.triggered.connect(self.get_tablewidget_filenode_linkattr)
        # menuitemB.triggered.connect(functools.partial(self.add_gpucache_node,"gpu"))
    def show_custom_menu(self):
        self.tablewidgetRightMenu.exec_(QtGui.QCursor().pos())
    #listwidget 右键
    def create_listwidget_right_menu(self):
        add_icon = QtGui.QIcon()
        add_icon.addPixmap(QtGui.QPixmap(":/UVTBAdd.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        remove_icon = QtGui.QIcon()
        remove_icon.addPixmap(QtGui.QPixmap(":/UVTBRemove.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        save_icon = QtGui.QIcon()
        save_icon.addPixmap(QtGui.QPixmap(":/save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        open_icon = QtGui.QIcon()
        open_icon.addPixmap(QtGui.QPixmap(":/fileOpen.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)


        self.listwidgetRightMenu = QtWidgets.QMenu(self)
        menuitemA = self.listwidgetRightMenu.addAction(add_icon, u'添加')
        menuitemB = self.listwidgetRightMenu.addAction(remove_icon, u'删除')
        self.listwidgetRightMenu.addSeparator()
        menuitemC = self.listwidgetRightMenu.addAction(save_icon, u'保存预设')
        menuitemD = self.listwidgetRightMenu.addAction(open_icon, u'加载预设')


        menuitemA.triggered.connect(self.create_new_listwidgetitem)
        menuitemB.triggered.connect(self.delete_listwidgetitem)
        menuitemC.triggered.connect(self.save_default_name)
        menuitemD.triggered.connect(functools.partial(self.show_setting_win, u'设置命名预设', self.new_name_set_json, self.new_name_set, self))
    def show_listwidget_right_menu(self):
        self.listwidgetRightMenu.exec_(QtGui.QCursor().pos())
    def get_text(self):
        text, okPressed = QtWidgets.QInputDialog.getText(self, u"新建标签",u"名字:", QtWidgets.QLineEdit.Normal, "")
        if okPressed and text != '':
            return text
        else:
            return None
    # 创建listwidgetitem
    def create_new_listwidgetitem(self):
        itemname = self.get_text()
        if itemname:
            self.name_listwidget.addItem(itemname)
            self.refresh_name_label()
    # 删除listwidgetitem
    def delete_listwidgetitem(self):
        selitem = self.name_listwidget.selectedItems()
        if selitem:
            for sel in selitem:
                row = self.name_listwidget.row(sel)
                self.name_listwidget.takeItem(row)
        self.refresh_name_label()
    # 保存预设
    def save_default_name(self):
        name = self.get_text()
        if name:
            _list = []
            for row in range(self.name_listwidget.count()):
                item = self.name_listwidget.item(row)
                _list.append(item.text())
            if name in self.new_name_set:
                if self.show_warning_window(u"警告", u"是否覆盖预设??") == QtWidgets.QMessageBox.Yes:
                    self.new_name_set[name] = _list
            else:
                self.new_name_set[name] = _list
            self.save_json_dic()
            self.reload_json_dic()
    # 将listwidget所有item相加，返回一个string
    def get_filepath_name(self):
        filename = ""
        for row in range(self.name_listwidget.count()):
            listwidget_item = self.name_listwidget.item(row)
            filename += listwidget_item.text() + "_"
        return filename[:-1]
    # 刷新listwidget上面的lable
    def refresh_name_label(self):
        filename = self.get_filepath_name() + ".png"
        self.pic_label.setText(filename)
        self.set_tablewidget_filename()
        pass
    # 显示路径ui
    def change_path_show(self):
        if self.pic_path_radioButton.isChecked() and not self.new_path_radioButton.isChecked():
            self.save_filepath = None
            self.widget_2.setVisible(False)
        elif not self.pic_path_radioButton.isChecked() and self.new_path_radioButton.isChecked():
            self.save_filepath = None
            self.widget_2.setVisible(True)
    # 选择路径
    def get_filepath(self):
        filename = QtWidgets.QFileDialog.getExistingDirectory(self, u'选择图片另存路径', './')
        if filename and os.path.exists(filename) and os.path.isdir(filename):
            return filename
        else:
            return None
    # 获取选择路径
    def change_line_edit(self):
        filepath = self.get_filepath()
        if filepath:
            self.path_lineedit.setText(filepath)
            self.save_filepath = filepath

#--------------------------------------------------------------------------------------------------ui设置
    # 重新加载json文件
    def reload_json_dic(self):
        if os.path.exists(self.node_fileattr_josn):
            with open(self.node_fileattr_josn, "r") as f:
                self.node_fileattr = json.load(f)
        if os.path.exists(self.shader_nodetype_josn):
            with open(self.shader_nodetype_josn, "r") as f:
                self.shader_nodetype = json.load(f)
        if os.path.exists(self.attr_filename_json):
            with open(self.attr_filename_json, "r") as f:
                self.attr_filename = json.load(f)
        if os.path.exists(self.new_name_set_json):
            with open(self.new_name_set_json, "r") as f:
                self.new_name_set = json.load(f)
    # 保存json文件
    def save_json_dic(self):
        with open(self.node_fileattr_josn, "w") as f:
            json.dump(self.node_fileattr,f,indent=4)
        with open(self.shader_nodetype_josn, "w") as f:
            json.dump(self.shader_nodetype,f,indent=4)
        with open(self.attr_filename_json, "w") as f:
            json.dump(self.attr_filename,f,indent=4)
        with open(self.new_name_set_json, "w") as f:
            json.dump(self.new_name_set,f,indent=4)
    # 预设窗口
    def show_setting_win(self, title, json_path, json_dic, parent_widget):
        ui = Default_Setting.Set_setting_info(title, json_path, json_dic, parent_widget)
        if ui:
            self.reload_json_dic()

    # 获取选择模型的sg节点
    def get_selobj_sg(self):
        selobjs = cmds.ls(sl=1,l=1)
        sg_node_list = []
        if not selobjs:
            self.show_warning_window(u"警告", u"你还没有选择物体")
            return
        for obj in selobjs:
            shapes = cmds.listRelatives(obj, c=1, pa=1)
            if shapes:
                for shape in shapes:
                    sg_node_list = sg_node_list + cmds.listConnections(shape, s=0, d=1)
        return list(set(sg_node_list))
    # merge字典
    def merge_dict(self, x, y):
        if x and not y:
            return x
        elif not x and y:
            return y
        elif x and y:
            for k,v in y.items():
                if k in x.keys():
                    if not v in x[k]:
                        x[k] += v
                else:
                    x[k] = v
            return x
    # 递归获取sg节点链接的所有贴图节点
    def get_sg_sourse_filenode(self,sgnode):
        filenode_filepath_dic = {}
        source_nodes = cmds.listConnections(sgnode, s=1, d=0)
        for node in source_nodes:
            nodetype = cmds.objectType(node)
            # 节点类型是否是需要的
            if nodetype in self.node_fileattr:
                filenode_filepath_dic[node] = cmds.getAttr(node + self.node_fileattr[nodetype])
            # 是否有上游节点
            if cmds.listConnections(node, s=1, d=0):
                _dic = self.get_sg_sourse_filenode(node)
                filenode_filepath_dic = self.merge_dict(filenode_filepath_dic, _dic)
        return filenode_filepath_dic
    # 返回item在列表的第几个
    def return_list_index(self, _list, value):
        for index,val in enumerate(_list):
            if val == value:
                return index
        return None
    # 设置表格某一行的内容
    def set_tablewidget_row(self,_widget, row, string_list):
        column = 0
        for item in string_list:
            if item:
                newItem=QtWidgets.QTableWidgetItem(str(item))
                _widget.setItem(row, column, newItem)
            column += 1
    # 获取表格的某一行的内容
    def get_tablewidget_row(self, _widget ,row):
        _dic = {}
        for column in range(_widget.columnCount()):
            item = _widget.item(row, column)
            if item:
                _dic[column] = item.text()
        return _dic
    # 获取选择的模型的贴图
    def get_sel_textures(self):
        self.attr_tablewidget.clearContents()

        sel_sg = self.get_selobj_sg()
        filenode_filepath_dic = {}
        if sel_sg:
            for sg in sel_sg:
                all_dic = self.get_sg_sourse_filenode(sg)
                filenode_filepath_dic = self.merge_dict(filenode_filepath_dic, all_dic)
            row = 0
            if filenode_filepath_dic == {} or not filenode_filepath_dic:
                cmds.confirmDialog( title=u'错误', message=u"未找到贴图节点，\n请检查是否将插件节点添加到脚本识别",button=[u"继续"])
                return
            self.attr_tablewidget.setRowCount(len(filenode_filepath_dic))
            for filenode,filepath in filenode_filepath_dic.items():
                self.set_tablewidget_row(self.attr_tablewidget, row, [filenode,filepath])
                row += 1

    # 返回2个节点链接的属性
    def return_nods_link_attr(self,up_node,down_node):
        _dic = {up_node:[],down_node:[]}
        
        node_attr_list = cmds.listConnections(up_node, s=0, d=1, c=1)
        for index,val in enumerate(node_attr_list):
            if val == down_node:
                _dic[up_node].append(node_attr_list[index-1].split(".",1)[-1])
        
        node_attr_list = cmds.listConnections(down_node, s=1, d=0, c=1)
        for index,val in enumerate(node_attr_list):
            if val == up_node:
                _dic[down_node].append(node_attr_list[index-1].split(".",1)[-1])
        return _dic
    # 获取贴图节点链接的材质节点
    def get_filenode_link_shader(self,filenode):
        destination_nodes = cmds.listConnections(filenode, s=0, d=1)
        for node in destination_nodes:
            nodetype = cmds.objectType(node)
            # 节点类型是否是是材质
            if nodetype in self.shader_nodetype["shader"]:
                linkattr = self.return_nods_link_attr(filenode, node)
                if len(linkattr[node]) == 1:
                    return linkattr[node][0]
        # 是否有下游节点
        for node in destination_nodes:
            if cmds.listConnections(node, s=0, d=1):
                return self.get_filenode_link_shader(node)
        return None
    # 获取贴图链接的属性
    def get_tablewidget_filenode_linkattr(self):
        for row in range(self.attr_tablewidget.rowCount()):
            filenode = self.attr_tablewidget.item(row,0).text()
            # 链接到材质球上的属性
            attr = self.get_filenode_link_shader(filenode)
            # 对应的文件名
            name = None
            if attr in self.attr_filename:
                name = self.attr_filename[attr]
            self.set_tablewidget_row(self.attr_tablewidget, row, [None,None,attr,name])
        self.refresh_name_label()
    # 获取文件命名,填到左侧表格
    def set_tablewidget_filename(self):
        for row in range(self.attr_tablewidget.rowCount()):
            filefullpath = self.attr_tablewidget.item(row,1)
            attrname = self.attr_tablewidget.item(row,3)
            if filefullpath and attrname:
                file_type = filefullpath.text().rsplit(".",1)[-1]
                filename = self.get_filepath_name() + "." + file_type
                if "Attr" in filename:
                    filename = filename.replace("Attr",attrname.text())
                self.set_tablewidget_row(self.attr_tablewidget, row, [None,None,None,None,filename])

    # 获取双击前的内容
    def get_doubleclick_content(self, _widgetItem):
        if _widgetItem:
            self.filenode_oldname = self.get_tablewidget_row(self.attr_tablewidget, _widgetItem.row())
    # 节点和路径更改，更改节点
    def rename_filenode_and_attr(self, _widgetItem):
        if _widgetItem:
            column = _widgetItem.column()
            row = _widgetItem.row()
            if self.filenode_oldname:
                if column == 0 and column in self.filenode_oldname:
                    cmds.rename(self.filenode_oldname[0], _widgetItem.text())
                elif column == 1 and column in self.filenode_oldname:
                    if cmds.objExists(self.filenode_oldname[0]):
                        file_type = cmds.objectType(self.filenode_oldname[0])
                        attr = self.node_fileattr[file_type]
                        if os.path.exists(_widgetItem.text()):
                            cmds.setAttr((self.filenode_oldname[0] + attr), _widgetItem.text(), type="string")
        self.filenode_oldname = {}
        pass

#--------------------------------------------------------------------------------------------------apply
    # 获取UDIM所有图片
    def get_udim_all_file(self, file_fullpath):
        file_list = []
        array = file_fullpath.replace("\\","/").rsplit("/",1)
        file_path = array[0]
        file_name = array[-1]
        num = re.findall("\d{4}", file_name)
        if num:
            find_text = file_name.split(num[0])[0]
            for filename in os.listdir(file_path):
                if re.findall(find_text, filename):
                    fullpath = file_path + "/" + filename
                    if os.path.splitext(fullpath)[-1] != ".tx":
                        file_list.append(fullpath)
                    else:
                        file_list.append(file_fullpath)
        return file_list
    # UDIM改名
    def return_udim_newname(self, file_fullpath, newname, new_fullpath):
        udim_list = self.get_udim_all_file(file_fullpath)
        newpath = new_fullpath.rsplit("/",1)[0]
        _dic = {}
        for fullpath in udim_list:
            houzhui = re.findall(".\d{4}.\w{3}",fullpath)
            if houzhui:
                _dic[fullpath] = newpath + "/" + newname + houzhui[-1]
        return _dic
    # 警告提示
    def show_warning_window(self, title, message):
        return QtWidgets.QMessageBox.warning(self, title, message, QtWidgets.QMessageBox.Yes)
    # 检查命名是否有Attr
    def check_Attr_in_newfilename(self):
        for row in range(self.name_listwidget.count()):
            listwidget_item = self.name_listwidget.item(row)
            if listwidget_item.text() == "Attr":
                return True
        return False
    # 检查属性命名是否都有替换
    def check_attr_to_name_exitis(self):
        if self.check_Attr_in_newfilename():# 如果有属性，
            for row in range(self.attr_tablewidget.rowCount()):
                attrname = self.attr_tablewidget.item(row,3)
                if not attrname:
                    return False
        return True
    # 检查是否会覆盖原文件
    def check_oldfile_cover(self):
        old_file_list = []
        new_file_list = []
        for row in range(self.attr_tablewidget.rowCount()):
            item1 = self.attr_tablewidget.item(row,1)
            item3 = self.attr_tablewidget.item(row,3)
            item4 = self.attr_tablewidget.item(row,4)
            if item1 and item3 and item4:
                old_fullpath = item1.text()
                attr_name = item3.text()
                new_fullpath = None
                # 路径处理
                if self.pic_path_radioButton.isChecked() and not self.new_path_radioButton.isChecked():
                    new_fullpath = old_fullpath.rsplit("/",1)[0] + "/" +item4.text()
                elif not self.pic_path_radioButton.isChecked() and self.new_path_radioButton.isChecked():
                    # 新路径
                    if not self.save_filepath:
                        self.show_warning_window(u"警告", u"请选择要保存的路径")
                        return
                    else:
                        new_fullpath = self.save_filepath + "/" +item4.text()

                # new_fullpath = old_fullpath.rsplit("/",1)[0] + "/" +item4.text()
                old_file_list.append(old_fullpath)
                new_file_list.append(new_fullpath)
        for fullpath in new_file_list:
            if fullpath in old_file_list:
                self.show_warning_window(u"警告", u"检测到旧文件会被覆盖\n未防止意外发生\n请修贴图保存路径\n\n程序将停止运行")
                return False
            if os.path.exists(fullpath):
                self.show_warning_window(u"警告", u"检测到新命名文件中有文件已存在\n未防止意外发生\n请修贴图保存路径\n\n程序将停止运行")
                return False
        if len(new_file_list) != len(set(new_file_list)):
            self.show_warning_window(u"警告", u"检测到新命名文件中有重复文件\n后面生成的文件将直接使用之前已经命名好的文件\n\n程序将继续运行")
            return True

        return True
    # 检查是否目标文件是否重名
    def check_newfile_name(self):
        _list = []
        for row in range(self.attr_tablewidget.rowCount()):
            item4 = self.attr_tablewidget.item(row,4)
            if item4:
                if item4.text() in _list:
                    return False
                else:
                    _list.append(item4.text())
        return True
    # file节点设置
    def set_filenode(self, filenode, old_fullpath, new_fullpath, newtext):
        # 设置file节点
        try:
            cmds.setAttr((filenode + ".ignoreColorSpaceFileRules"),1)
        except:
            pass
        # 复制文件
        if cmds.objectType(filenode) == "file" and cmds.getAttr(filenode + ".uvTilingMode") == 3:
            _dic = self.return_udim_newname(old_fullpath, newtext, new_fullpath)
            for k,v in _dic.items():
                shutil.copy(k, v)
            # 指定新路径
            cmds.setAttr((filenode + ".ftn"),_dic.values()[0],type="string")
        else:
            shutil.copy(old_fullpath, new_fullpath)
            # 指定新路径
            cmds.setAttr((filenode + ".ftn"),new_fullpath,type="string")
    
    # 重命名
    def start_rename(self):
        # 如果右侧有属性，且有属性未设置替换命名
        if not self.check_attr_to_name_exitis():
            self.show_warning_window(u"警告", u"有属性未设置替换命名\n将停止运行程序")
            return
        # 新文件是否会覆盖旧文件
        if not self.check_oldfile_cover():
            self.show_warning_window(u"警告", u"新文件会覆盖旧文件\n将停止运行程序")
            return
        # 检查目标文件是否重名
        if not self.check_newfile_name():
            self.show_warning_window(u"警告", u"新命名的文件有重名\n将停止运行程序")
            return
        #
        for row in range(self.attr_tablewidget.rowCount()):
            item0 = self.attr_tablewidget.item(row,0)
            item1 = self.attr_tablewidget.item(row,1)
            item4 = self.attr_tablewidget.item(row,4)
            if item0 and item1 and item4:
                filenode = item0.text()
                old_fullpath = item1.text()
                new_fullpath = None
                # 路径处理
                if self.pic_path_radioButton.isChecked() and not self.new_path_radioButton.isChecked():
                    new_fullpath = old_fullpath.rsplit("/",1)[0] + "/" +item4.text()
                elif not self.pic_path_radioButton.isChecked() and self.new_path_radioButton.isChecked():
                    # 新路径
                    if not self.save_filepath:
                        self.show_warning_window(u"警告", u"请选择要保存的路径")
                        return
                    else:
                        new_fullpath = self.save_filepath + "/" +item4.text()
                self.set_filenode(filenode, old_fullpath, new_fullpath, item4.text().rsplit(".",1)[0])


            








class Show_RenameShaderWindow(RenameShaderWindow,QtWidgets.QMainWindow):
    def __init__(self,parent=None):
        RenameShaderWindow.__init__(self)
        QtWidgets.QMainWindow.__init__(self,parent = mayaMainWindows())
        self.setupUi(self)
        self.retranslateUi(self)

        self.update_title(1.09)


try:
    renameshaderwin.close()
    renameshaderwin.deleteLater()
except:
    pass
renameshaderwin = Show_RenameShaderWindow()
renameshaderwin.show()
